class Quad2D
{

  PVector[] points = new PVector[4];
  
  PVector center;
  
  Quad2D()
  {
     points[0] = new PVector(0,0,0);
     points[1] = new PVector(100,0,0);
     points[2] = new PVector(100,100,0);
     points[3] = new PVector(0,100,0);
     
      SetCenter();
  }
  
  Quad2D(PVector p1, PVector p2, PVector p3, PVector p4)
  {
     points[0] = p1;
     points[1] = p2;
     points[2] = p3;
     points[3] = p4;    
     
     SetCenter();
  }
  
  Quad2D(PVector iPos, float iWidth, float iHeight)
  {
     points[0] = new PVector(iPos.x - iWidth/2, iPos.y - iHeight/2, iPos.z );
     points[1] = new PVector(iPos.x + iWidth/2, iPos.y - iHeight/2, iPos.z );
     points[2] = new PVector(iPos.x + iWidth/2, iPos.y + iHeight/2, iPos.z );
     points[3] = new PVector(iPos.x - iWidth/2, iPos.y + iHeight/2, iPos.z );
     
     center = new PVector();
     center.set(iPos);
  }
  
  void set(Quad2D inQuad)
  {
    points[0].set(inQuad.points[0]);
    points[1].set(inQuad.points[1]);
    points[2].set(inQuad.points[2]);
    points[3].set(inQuad.points[3]);
    center.set(inQuad.center);
  }
  
  void SetCenter() //positions the center at the average of all points
  {
    PVector avg = new PVector(0,0,0);
    
    for (int i = 0; i < 4; i++)
            avg.add(points[i]);
            
    avg.div(4);
    
    center = avg;
  }
  
  void Translate(PVector change)
  {
     points[0].add(change);
     points[1].add(change);
     points[2].add(change);
     points[3].add(change);
     
     center.add(change);
  }
  
  void MoveTo(PVector dest)
  {
    PVector change = PVector.sub(dest, center);
    
    Translate(change);
    
//    float iWidth = points[1].x - points[0].x;
//    float iHeight = points[2].y - points[1].y;
//    
//    points[0] = new PVector(dest.x - iWidth/2, dest.y - iHeight/2, dest.z );
//    points[1] = new PVector(dest.x + iWidth/2, dest.y - iHeight/2, dest.z );
//    points[2] = new PVector(dest.x + iWidth/2, dest.y + iHeight/2, dest.z );
//    points[3] = new PVector(dest.x - iWidth/2, dest.y + iHeight/2, dest.z );
//     
//    center.set(dest);
  }
  
  void Draw(PImage img)
  {
    beginShape(QUADS);
    texture(img);
    
    vertex(points[0].x, points[0].y, points[0].z, 0, 0);
    vertex(points[1].x, points[1].y, points[1].z, img.width, 0);
    vertex(points[2].x, points[2].y, points[2].z, img.width, img.height);
    vertex(points[3].x, points[3].y, points[3].z, 0, img.height);
    
    endShape();
  }
  
  Quad2D RotateQuad(PVector rotAxis, float rotAmount)
  {
    Quad2D returnQuad = new Quad2D();
    returnQuad.points[0] = RotateVector(rotAxis, rotAmount, points[0]);
    returnQuad.points[1] = RotateVector(rotAxis, rotAmount, points[1]);
    returnQuad.points[2] = RotateVector(rotAxis, rotAmount, points[2]);
    returnQuad.points[3] = RotateVector(rotAxis, rotAmount, points[3]);
    
    returnQuad.center = RotateVector(rotAxis, rotAmount, center);
    
    return returnQuad;
  }
  
  PVector  RotateVector(PVector rotAxis, float rotAmount, PVector rotVec)
  {
    PVector rotateVec = new PVector();
    rotateVec.set(rotVec);
    rotateVec.sub(rotAxis);
    PVector returnVec = new PVector();
    
    returnVec.x = (rotateVec.x *(cos(rotAmount))) - (rotateVec.y *(sin(rotAmount)));
    returnVec.y = (rotateVec.x *(sin(rotAmount))) + (rotateVec.y *(cos(rotAmount)));
    returnVec.add(rotAxis);
    return returnVec;
  }
  
  void MirrorQuad(PVector axisA, PVector axisB)
  {
    points[0] = MirrorVector2D( points[0], axisA, axisB );
    points[1] = MirrorVector2D( points[1], axisA, axisB );
    points[2] = MirrorVector2D( points[2], axisA, axisB );
    points[3] = MirrorVector2D( points[3], axisA, axisB );
    
    center = MirrorVector2D( center, axisA, axisB );
  }
  
   PVector MirrorVector2D(PVector v1, PVector axisA, PVector axisB)
  {
    PVector axis = PVector.sub(axisA, axisB);
    
    PVector axisNormal = ReturnNormal(axis, true);
    
    PVector mirrorVec = new PVector();
    
    mirrorVec.set(v1);
    mirrorVec.sub(axisB);
    mirrorVec = PVector.sub(mirrorVec, PVector.div( PVector.mult( axisNormal, 2*mirrorVec.dot(axisNormal) ), pow(axisNormal.mag(),2))     );
    mirrorVec.add(axisB);
      
     return mirrorVec;
  }
  
  PVector ReturnNormal(PVector v1, boolean clockwise)
  {
    PVector normalV1 = new PVector();
    normalV1.set(v1);

    if (clockwise)
    {
      float y = normalV1.y;
      normalV1.y = -normalV1.x;
      normalV1.x = y;
    }
    else
    {
      float y = normalV1.y;
      normalV1.y = normalV1.x;
      normalV1.x = -y;
    }

    return normalV1;
  }

}
